"""Entry point for workflow_scaffolding module.

Allows running as: python -m workflow_scaffolding

Copyright 2025 Smart-AI-Memory
Licensed under Fair Source License 0.9
"""

from .cli import main

if __name__ == "__main__":
    main()
